from django.db.models import (
    CASCADE,
    PROTECT,
    CharField,
    DateTimeField,
    FileField,
    ForeignKey,
    Model,
    TextField,
)

from function_tools.models import (
    RegisteredFunction,
)
from m3_db_utils.models import (
    IntegerModelEnum,
)

from web_bb.core.documents.enums import (
    DocumentEnum,
)


class BaseElectronicDocumentStatus(IntegerModelEnum):
    """
    Базовая модель-перечесление статусов электронных документов
    """

    class Meta:
        abstract = True


class BaseElectronicSignatureStatus(IntegerModelEnum):
    """
    Базовая модель-перечисление статусов электронных подписей
    """

    class Meta:
        abstract = True


class BaseElectronicDocumentKind(Model):
    """
    Базовая модель вида документа
    """

    name = CharField(verbose_name='Наименование вида документа', max_length=512)
    pack_code = CharField(
        verbose_name='Пак вида документа',
        max_length=128,
        choices=DocumentEnum.get_choices(),
        blank=True,
        null=True,
    )

    class Meta:
        abstract = True


class BaseElectronicDocument(Model):
    """
    Базовая модель электронного документа
    """

    # Поле должно быть переопределено у потомка с использованием реальной модели статуса наследника
    # BaseElectronicDocumentStatus
    status = ForeignKey(
        to=BaseElectronicDocumentStatus,
        verbose_name='Статус',
        on_delete=PROTECT,
    )

    number = CharField(
        verbose_name='Номер',
        max_length=1024,
    )
    date = DateTimeField(
        verbose_name='Дата',
    )

    # Поле должно быть переопределено у потомка с использованием реальной модели статуса наследника
    # BaseElectronicDocumentKind
    document_kind = ForeignKey(
        to=BaseElectronicDocumentKind,
        verbose_name='Вид документа',
        on_delete=PROTECT,
    )

    class Meta:
        abstract = True


class BaseElectronicDocumentEditingLog(Model):
    """
    Базовая модель хранения лога смены статуса электронного документа
    """

    # Поле должно быть переопределено у потомка с использованием реальной модели статуса наследника
    # BaseElectronicDocument
    document = ForeignKey(
        to=BaseElectronicDocument,
        verbose_name='Электронный документ',
        on_delete=CASCADE,
    )

    user = ForeignKey(to='auth.User', verbose_name='Пользователь', on_delete=PROTECT)

    # Поле должно быть переопредено у потомка с использованием реальной модели статуса наследника
    # BaseElectronicDocumentStatus
    from_status = ForeignKey(
        to=BaseElectronicDocumentStatus,
        verbose_name='Статус до внесения изменений',
        on_delete=PROTECT,
        null=True,
        blank=True,
    )

    # Поле должно быть переопредено у потомка с использованием реальной модели статуса наследника
    # BaseElectronicDocumentStatus
    to_status = ForeignKey(
        to=BaseElectronicDocumentStatus,
        verbose_name='Статус после внесения изменений',
        on_delete=PROTECT,
    )

    notice = TextField(
        verbose_name='Примечание',
        max_length=1024,
    )

    created_at = DateTimeField(verbose_name='Дата и время внесения изменений', auto_now_add=True)

    class Meta:
        abstract = True


class BaseElectronicDocumentAttachment(Model):
    """
    Связанный файл
    """

    # Поле должно быть переопределено у потомка с использованием реальной модели статуса наследника
    # BaseElectronicDocument
    document = ForeignKey(
        to=BaseElectronicDocument,
        verbose_name='Электронный документ',
        on_delete=CASCADE,
    )

    # Поле лучше переопределить у потомка с указанием корректного места сохранения файла
    link = FileField(
        verbose_name='Загружаемый файл',
        upload_to='edm/%Y/%m/%d',
    )

    created_at = DateTimeField(
        verbose_name='Дата и время создания',
        auto_now_add=True,
    )

    class Meta:
        abstract = True


class BaseElectronicDocumentElectronicSignature(Model):
    """
    Электронная подпись электронного документа
    """

    # Поле должно быть переопределено у потомка с использованием реальной модели статуса наследника
    # BaseElectronicDocument
    document = ForeignKey(
        to=BaseElectronicDocument,
        verbose_name='Электронный документ',
        on_delete=CASCADE,
    )

    # Поле должно быть переопределено у потомка с использованием реальной модели статуса наследника
    # BaseElectronicSignatureStatus
    status = ForeignKey(
        to=BaseElectronicSignatureStatus,
        verbose_name='ЭП',
        on_delete=PROTECT,
    )

    signed_at = DateTimeField(
        verbose_name='Дата и время подписания',
        auto_now_add=True,
    )

    signer = ForeignKey(
        to='auth.User',
        verbose_name='Подписант',
        on_delete=PROTECT,
    )

    # Поле лучше переопределить у потомка с указанием корректного места сохранения файла
    file_link = FileField(
        verbose_name='Подписанный файл',
        upload_to='edm_signed_files/%Y/%m/%d',
    )

    notice = TextField(
        verbose_name='Комментарий',
        max_length=1024,
    )

    class Meta:
        abstract = True


class BaseElectronicDocumentTransition(Model):
    """
    Переход электронного документа в новый
    """

    # Поле должно быть переопределено у потомка с использованием реальной модели статуса наследника
    # BaseElectronicDocumentStatus
    from_status = ForeignKey(
        to=BaseElectronicDocumentStatus,
        verbose_name='Начальный статус',
        on_delete=PROTECT,
    )

    # Поле должно быть переопределено у потомка с использованием реальной модели статуса наследника
    # BaseElectronicDocumentStatus
    to_status = ForeignKey(
        to=BaseElectronicDocumentStatus,
        verbose_name='Конечный статус',
        on_delete=PROTECT,
    )

    function = ForeignKey(
        to=RegisteredFunction,
        verbose_name='Функция осуществляющая переход',
        on_delete=PROTECT,
    )

    # Поле должно быть переопределено у потомка с использованием реальной модели статуса наследника
    # BaseElectronicDocumentKind
    document_kind = ForeignKey(
        to=BaseElectronicDocumentKind,
        verbose_name='Вид документа',
        on_delete=PROTECT,
    )

    class Meta:
        abstract = True
